@@ -154,6 +154,9 @@ private:
 private:
   void Uninitialize();
 
+public:
+  void Close() override;
+
 public:
   void Reset() override;
 
@@ -679,15 +682,7 @@ public:
     return theSession.lock();
   }
 
-public:
-  static std::shared_ptr<SessionImpl> GetSession()
-  {
-    if (theSession.expired())
-    {
-      MIKTEX_UNEXPECTED();
-    }
-    return theSession.lock();
-  }
+#define SESSION_IMPL() (MiKTeX::ABF3880A6239B84E87DC7E727A8BBFD4::SessionImpl::theSession.expired() ? (MIKTEX_UNEXPECTED(), nullptr) : MiKTeX::ABF3880A6239B84E87DC7E727A8BBFD4::SessionImpl::theSession.lock())
 
 public:
   MiKTeX::Util::PathName GetRelativeFilenameDatabasePathName(unsigned r);
