@@ -54,6 +54,19 @@
 #include "RootDirectoryInfo.h"
 #include "VersionNumber.h"
 
+/// Gets the current session object.
+/// @return Returns a smart pointer to the current session object.
+#define MIKTEX_SESSION() ([]()                      \
+{                                                   \
+    auto session = MiKTeX::Core::Session::TryGet(); \
+    if (session == nullptr)                         \
+    {                                               \
+        MIKTEX_UNEXPECTED();                        \
+    }                                               \
+    return session;                                 \
+}                                                   \
+())
+
 /// @namespace MiKTeX::Core
 /// @brief The core namespace.
 MIKTEX_CORE_BEGIN_NAMESPACE;
@@ -540,10 +553,6 @@ public:
     FileAccess access = FileAccess::None;
   };
 
-  /// Gets the current session object.
-  /// @return Returns a smart pointer to the current session object.
-  static MIKTEXCORECEEAPI(std::shared_ptr<Session>) Get();
-
   /// Tries to get the current session object.
   /// @return Returns a smart pointer to the current session object. Can be `nullptr`.
   static MIKTEXCORECEEAPI(std::shared_ptr<Session>) TryGet();
@@ -555,6 +564,9 @@ public:
 
   virtual MIKTEXTHISCALL ~Session() noexcept = 0;
 
+  /// Closes this session object.
+  virtual void MIKTEXTHISCALL Close() = 0;
+
   /// Resets this session object.
   virtual void MIKTEXTHISCALL Reset() = 0;
 
@@ -1180,7 +1192,10 @@ MIKTEX_CORE_END_NAMESPACE;
 #define MIKTEX_FATAL_ERROR_5(message, description, remedy, tag, ...) \
   MiKTeX::Core::Session::FatalMiKTeXError(message, description, remedy, tag, MiKTeX::Core::MiKTeXException::KVMAP(__VA_ARGS__), MIKTEX_SOURCE_LOCATION())
 
-#define MIKTEX_INTERNAL_ERROR() MIKTEX_FATAL_ERROR(MIKTEXTEXT("MiKTeX encountered an internal error."))
+#define MIKTEX_STRINGIFY_(x) #x
+#define MIKTEX_STRINGIFY(x) MIKTEX_STRINGIFY_(x)
+
+#define MIKTEX_INTERNAL_ERROR() MIKTEX_FATAL_ERROR(__FILE__ ":" MIKTEX_STRINGIFY(__LINE__) ": internal error")
 
 #define MIKTEX_UNEXPECTED() MIKTEX_INTERNAL_ERROR()
 
