@@ -1,6 +1,6 @@
 /* miktex/TeXAndFriends/TeXMFMemoryHandlerImpl.h:       -*- C++ -*-
 
-   Copyright (C) 2017-2020 Christian Schenk
+   Copyright (C) 2017-2021 Christian Schenk
 
    This file is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
@@ -22,7 +22,7 @@
 #if !defined(FEFFF218B53147ED8CDE64F68A13D234)
 #define FEFFF218B53147ED8CDE64F68A13D234
 
-#include <miktex/Core/ConfigNames>
+#include <miktex/Configuration/ConfigNames>
 
 #include <miktex/TeXAndFriends/config.h>
 
@@ -61,10 +61,10 @@ protected:
   int GetConfigValue(const std::string& valueName, int defaultValue) const
   {
     std::shared_ptr<MiKTeX::Core::Session> session = texmfapp.GetSession();
-    int value = session->GetConfigValue(MIKTEX_CONFIG_SECTION_NONE, valueName, MiKTeX::Core::ConfigValue(-1)).GetInt();
+    int value = session->GetConfigValue(MIKTEX_CONFIG_SECTION_NONE, valueName, MiKTeX::Configuration::ConfigValue(-1)).GetInt();
     if (value < 0)
     {
-      value = session->GetConfigValue(texmfapp.GetProgramName(), valueName, MiKTeX::Core::ConfigValue(defaultValue)).GetInt();
+      value = session->GetConfigValue(texmfapp.GetProgramName(), valueName, MiKTeX::Configuration::ConfigValue(defaultValue)).GetInt();
     }
     return value;
   }
