@@ -2,7 +2,7 @@
 ** DvisvgmSpecialHandler.cpp                                            **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -21,6 +21,7 @@
 #include <array>
 #include <cstring>
 #include <utility>
+#include "Calculator.hpp"
 #include "DvisvgmSpecialHandler.hpp"
 #include "InputBuffer.hpp"
 #include "InputReader.hpp"
@@ -190,6 +191,37 @@ static void expand_constants (string &str, SpecialActions &actions) {
 }
 
 
+/** Evaluates substrings of the form {?(expr)} where 'expr' is a math expression,
+ *  and replaces the substring by the computed value.
+ *  @param[in,out] str string to scan for expressions */
+static void evaluate_expressions (string &str, SpecialActions &actions) {
+	size_t left = str.find("{?(");             // start position of expression macro
+	while (left != string::npos) {
+		size_t right = str.find(")}", left+2);  // end position of expression macro
+		if (right == string::npos)
+			break;
+		Calculator calc;
+		calc.setVariable("x", actions.getX());
+		calc.setVariable("y", actions.getY());
+		string expr = str.substr(left+3, right-left-3);  // math expression to evaluate
+		if (util::normalize_space(expr).empty())         // no expression given, e.g. {?( )}
+			str.erase(left, right-left+2);                // => replace with empty string
+		else {
+			try {
+				double val = calc.eval(expr);
+				XMLString valstr(val);
+				str.replace(left, right-left+2, valstr);
+				right = left+valstr.length()-1;
+			}
+			catch (CalculatorException &e) {
+				throw SpecialException(string(e.what())+" in '{?("+expr+")}'");
+			}
+		}
+		left = str.find("{?(", right+1);  // find next expression macro
+	}
+}
+
+
 /** Processes raw SVG fragments from the input stream. The SVG data must represent
  *  a single or multiple syntactically complete XML parts, like opening/closing tags,
  *  comments, or CDATA blocks. These must not be split and distributed over several
@@ -199,6 +231,7 @@ void DvisvgmSpecialHandler::processRaw (InputReader &ir, SpecialActions &actions
 	if (_nestingLevel == 0) {
 		string xml = ir.getLine();
 		if (!xml.empty()) {
+			evaluate_expressions(xml, actions);
 			expand_constants(xml, actions);
 			_pageParser.parse(xml, actions);
 		}
@@ -382,8 +415,8 @@ void DvisvgmSpecialHandler::dviPreprocessingFinished () {
 
 
 void DvisvgmSpecialHandler::dviEndPage (unsigned, SpecialActions &actions) {
-	_defsParser.flush(actions);
-	_pageParser.flush(actions);
+	_defsParser.finish(actions);
+	_pageParser.finish(actions);
 	actions.bbox().unlock();
 	for (auto &strvecpair : _macros) {
 		StringVector &vec = strvecpair.second;
@@ -406,8 +439,8 @@ vector<const char*> DvisvgmSpecialHandler::prefixes() const {
 /** Parses a fragment of XML code, creates corresponding XML nodes and adds them
  *  to the SVG tree. The code may be split and processed by several calls of this
  *  function. Incomplete chunks that can't be processed yet are stored and picked
- *  up again together with the next incoming XML fragment. If no further code should
- *  be appended, parameter 'finish' must be set.
+ *  up again together with the next incoming XML fragment. If a call of this function
+ *  is supposed to finish the parsing of an XML subtree, parameter 'finish' must be set.
  *  @param[in] xml XML fragment to parse
  *  @param[in] actions object providing the SVG tree functions
  *  @param[in] finish if true, no more XML is expected and parsing is finished */
@@ -416,59 +449,65 @@ void DvisvgmSpecialHandler::XMLParser::parse (const string &xml, SpecialActions
 	// incomplete tags are held back
 	_xmlbuf += xml;
 	size_t left=0, right;
-	while (left != string::npos) {
-		right = _xmlbuf.find('<', left);
-		if (left < right && left < _xmlbuf.length())  // plain text found?
-			(actions.svgTree().*_append)(util::make_unique<XMLText>(_xmlbuf.substr(left, right-left)));
-		if (right != string::npos) {
+	try {
+		while (left != string::npos) {
+			right = _xmlbuf.find('<', left);
+			if (left < right && left < _xmlbuf.length())  // plain text found?
+				(actions.svgTree().*_append)(util::make_unique<XMLText>(_xmlbuf.substr(left, right-left)));
+			if (right != string::npos) {
+				left = right;
+				if (_xmlbuf.compare(left, 9, "<![CDATA[") == 0) {
+					right = _xmlbuf.find("]]>", left+9);
+					if (right == string::npos) {
+						if (finish) throw SpecialException("expected ']]>' at end of CDATA block");
+						break;
+					}
+					(actions.svgTree().*_append)(util::make_unique<XMLCData>(_xmlbuf.substr(left+9, right-left-9)));
+					right += 2;
+				}
+				else if (_xmlbuf.compare(left, 4, "<!--") == 0) {
+					right = _xmlbuf.find("-->", left+4);
+					if (right == string::npos) {
+						if (finish) throw SpecialException("expected '-->' at end of comment");
+						break;
+					}
+					(actions.svgTree().*_append)(util::make_unique<XMLComment>(_xmlbuf.substr(left+4, right-left-4)));
+					right += 2;
+				}
+				else if (_xmlbuf.compare(left, 2, "<?") == 0) {
+					right = _xmlbuf.find("?>", left+2);
+					if (right == string::npos) {
+						if (finish) throw SpecialException("expected '?>' at end of processing instruction");
+						break;
+					}
+					(actions.svgTree().*_append)(util::make_unique<XMLText>(_xmlbuf.substr(left, right-left+2)));
+					right++;
+				}
+				else if (_xmlbuf.compare(left, 2, "</") == 0) {
+					right = _xmlbuf.find('>', left+2);
+					if (right == string::npos) {
+						if (finish) throw SpecialException("missing '>' at end of closing XML tag");
+						break;
+					}
+					closeElement(_xmlbuf.substr(left+2, right-left-2), actions);
+				}
+				else {
+					right = _xmlbuf.find('>', left+1);
+					if (right == string::npos) {
+						if (finish)	throw SpecialException("missing '>' or '/>' at end of opening XML tag");
+						break;
+					}
+					openElement(_xmlbuf.substr(left+1, right-left-1), actions);
+				}
+			}
 			left = right;
-			if (_xmlbuf.compare(left, 9, "<![CDATA[") == 0) {
-				right = _xmlbuf.find("]]>", left+9);
-				if (right == string::npos) {
-					if (finish)	throw SpecialException("expected ']]>' at end of CDATA block");
-					break;
-				}
-				(actions.svgTree().*_append)(util::make_unique<XMLCData>(_xmlbuf.substr(left+9, right-left-9)));
-				right += 2;
-			}
-			else if (_xmlbuf.compare(left, 4, "<!--") == 0) {
-				right = _xmlbuf.find("-->", left+4);
-				if (right == string::npos) {
-					if (finish)	throw SpecialException("expected '-->' at end of comment");
-					break;
-				}
-				(actions.svgTree().*_append)(util::make_unique<XMLComment>(_xmlbuf.substr(left+4, right-left-4)));
-				right += 2;
-			}
-			else if (_xmlbuf.compare(left, 2, "<?") == 0) {
-				right = _xmlbuf.find("?>", left+2);
-				if (right == string::npos) {
-					if (finish)	throw SpecialException("expected '?>' at end of processing instruction");
-					break;
-				}
-				(actions.svgTree().*_append)(util::make_unique<XMLText>(_xmlbuf.substr(left, right-left+2)));
-				right++;
-			}
-			else if (_xmlbuf.compare(left, 2, "</") == 0) {
-				right = _xmlbuf.find('>', left+2);
-				if (right == string::npos) {
-					if (finish)	throw SpecialException("missing '>' at end of closing XML tag");
-					break;
-				}
-				closeElement(_xmlbuf.substr(left+2, right-left-2), actions);
-			}
-			else {
-				right = _xmlbuf.find('>', left+1);
-				if (right == string::npos) {
-					if (finish)	throw SpecialException("missing '>' or '/>' at end of opening XML tag");
-					break;
-				}
-				openElement(_xmlbuf.substr(left+1, right-left-1), actions);
-			}
+			if (right != string::npos)
+				left++;
 		}
-		left = right;
-		if (right != string::npos)
-			left++;
+	}
+	catch (const SpecialException &e) {
+		_error = true;
+		throw;
 	}
 	if (left == string::npos)
 		_xmlbuf.clear();
@@ -514,7 +553,7 @@ void DvisvgmSpecialHandler::XMLParser::closeElement (const string &tag, SpecialA
 	if (_nameStack.empty())
 		throw SpecialException("spurious closing tag </" + name + ">");
 	if (_nameStack.back() != name)
-		throw SpecialException("expected </" + name + "> but found </" + _nameStack.back() + ">");
+		throw SpecialException("expected </" + _nameStack.back() + "> but found </" + name + ">");
 	(actions.svgTree().*_popContext)();
 	_nameStack.pop_back();
 }
@@ -522,9 +561,10 @@ void DvisvgmSpecialHandler::XMLParser::closeElement (const string &tag, SpecialA
 
 /** Processes any remaining XML fragments, checks for missing closing tags,
  *  and resets the parser state. */
-void DvisvgmSpecialHandler::XMLParser::flush (SpecialActions &actions) {
+void DvisvgmSpecialHandler::XMLParser::finish (SpecialActions &actions) {
 	if (!_xmlbuf.empty()) {
-		parse("", actions, true);
+		if (!_error)
+			parse("", actions, true);
 		_xmlbuf.clear();
 	}
 	string tags;
@@ -532,8 +572,8 @@ void DvisvgmSpecialHandler::XMLParser::flush (SpecialActions &actions) {
 		tags += "</"+_nameStack.back()+">, ";
 		_nameStack.pop_back();
 	}
-	if (!tags.empty()) {
+	if (!tags.empty() && !_error) {
 		tags.resize(tags.length()-2);
-		throw SpecialException("missing closing tags: "+tags);
+		throw SpecialException("missing closing tag(s): "+tags);
 	}
-}
\ No newline at end of file
+}
