@@ -765,8 +765,9 @@ void IniTeXMFApp::Finalize(bool keepSession)
   FlushPendingTraceMessages();
   packageInstaller = nullptr;
   packageManager = nullptr;
-  if (!keepSession)
+  if (session != nullptr && !keepSession)
   {
+    session->Close();
     session = nullptr;
   }
 }
