@@ -708,7 +708,7 @@ private:
   Options* options = nullptr;
 
 protected:
-  shared_ptr<Session> session = Session::Get();
+  shared_ptr<Session> session = MIKTEX_SESSION();
 };
 
 Driver::~Driver()
